package BBUVA;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Observable;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Sucursal extends Observable {

    private ArrayList<Producto> mProducto;
    private ArrayList<Remesa> mRemesa;
    private ArrayList<Empleado> mEmpleado;
    private ArrayList<Titular> mTitular;
    private ArrayList<Caja> mCaja;

    public Sucursal () {
        mProducto = new ArrayList<Producto>();
        mRemesa = new ArrayList<Remesa>();
        mTitular = new ArrayList<Titular>();
        mEmpleado = new ArrayList<Empleado>();
        mCaja = new ArrayList<Caja>();
    }

    public void addProducto(Producto val) {
        mProducto.add(val);
        this.setChanged();
        this.notifyObservers();
    }

    public void addTitular(Titular val) {
        mTitular.add(val);
        this.setChanged();
        this.notifyObservers();
    }

    public boolean anadirMovimientoCaja(float saldo, int numCaja, String descripcion) {
        if(this.getCaja(numCaja).getArq().getSaldoFinalTeorico()+saldo<0) {
            return false;
        }
        else {
            Movimiento m = new Movimiento();
            m.setCambioSaldo(saldo);
            m.setDescripcion(descripcion);
            Calendar actual= Calendar.getInstance();
            m.setFechaOperacion(actual.getTime());
            this.getCaja(numCaja).getArq().anadirMovimiento(m);
            return true;
        }
    }

    public String buscaNumeroNuevo() {
        ArrayList<String> lista=new ArrayList();
        for(int i=0;i<mProducto.size();i++)
            lista.add(mProducto.get(i).getNumeroProducto());

        String numero="20984002161";
        String aux="";

        for(long cont=0;cont<1000000000;cont++) {
            aux=Long.toString(cont);
            for(int i=aux.length();i<9;i++) aux="0"+aux;
            if(!lista.contains(numero+aux))break;
        }
        numero+=aux;
        return numero;
    }

    public boolean empleadoCorrecto(int numero,String clave) {
        for(int i=0;i<this.getMEmpleado().size();i++)
            if(this.getMEmpleado().get(i).getNumOperario()==numero && this.getMEmpleado().get(i).getClave().compareTo(clave)==0)
                return true;
        return false;
    }

    public int empleadoDirector(int numero, String clave) {
        if(this.empleadoCorrecto(numero,clave)) {
            if(this.getEmpleado(numero).isDirector()) {
                return 1;
            }
            return 0;
        }
        return -1;
    }

    public void enviarParteDiferencia(int numEmp, int numCaja, float saldoReal) {
        System.out.println("\nParte de diferencia:\n Saldo teórico: " + this.getCaja(numCaja).getArq().getSaldoFinal());
        System.out.println("\n Saldo real: " + saldoReal);
        this.getCaja(numCaja).getArq().setSaldoFinal(saldoReal);
        this.finalizarArqueo(numEmp, numCaja,false);
    }

    public Empleado getEmpleado(int numero) {
        for(int i=0;i<this.mEmpleado.size();i++)
            if(this.mEmpleado.get(i).getNumOperario()==numero)
                return this.mEmpleado.get(i);
        return null;
    }

    public Caja getCaja(int numero) {
        for(int i=0;i<this.mCaja.size();i++)
            if(this.mCaja.get(i).getIdCaja()==numero)
                return this.mCaja.get(i);
        return null;
    }

    public ArrayList<Producto> getProducto () {
        return mProducto;
    }

    public void incorporaRemesaAbono(String path) throws ParseException {
        try {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            String fechaRemesa="";
            String descripcion="";
            String numCta="";
            String numCtaOrd="";
            Remesa r=new Remesa();
            while ((linea = bf.readLine()) != null) {
                String buffer;
                buffer = linea.substring(0, 4);

                if(buffer.compareTo("5180")==0){
                    //registro cabecera presentador
                    r.setOrdenante(this.getTitular(linea.substring(28,68).trim()));
                }else if(buffer.compareTo("5380")==0){
                    fechaRemesa=linea.substring(22, 28);
                    numCtaOrd = linea.substring(68,88);
                }else if(buffer.compareTo("5680")==0){
                    OperRemesa op=new OperRemesa();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    op.setCambioSaldo(n.floatValue()*-1);
                    descripcion = linea.substring(114).trim();
                    op.setDescripcion(descripcion);
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    op.setFechaOperacion(df2.parse(fechaRemesa));
                    r.addOperRemesa(op);
                    //leo el numcta
                    numCta=linea.substring(68,88);
                    this.getProducto(numCta).getMovimiento().add(op);
                } else if(linea.substring(0,4).compareTo("5681")==0) {
                    descripcion += " " + linea.substring(28,67).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5682")==0) {
                    descripcion += "\n" + linea.substring(28,79).trim() + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5683")==0) {
                    descripcion += " " + linea.substring(28,68).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5684")==0) {
                    descripcion += "\n" + linea.substring(28,79) + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5685")==0) {
                    descripcion += " " + linea.substring(28,39) + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5686")==0) {
                    descripcion += "\n" + linea.substring(28).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                }else if(buffer.compareTo("5880")==0) {
                    Movimiento mov=new Movimiento();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    mov.setCambioSaldo(n.floatValue());
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    mov.setFechaOperacion(df2.parse(fechaRemesa));
                    mov.setDescripcion("Remesa " + df2.parse(fechaRemesa).toString());
                    this.getProducto(numCtaOrd).getMovimiento().add(mov);
                }
            }
            this.mRemesa.add(r);
            this.setChanged();
            this.notifyObservers();
        } catch (IOException ex) {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public boolean ingreso(String nCuenta,float importe,int caja) throws ParseException {
        Calendar actual= Calendar.getInstance();
        boolean retornar= this.anadirMovimiento(nCuenta,importe, "ingreso", actual.getTime());
        if(retornar)
            this.getCaja(caja).getArq().anadirMovimiento(this.getProducto(nCuenta).getMovimiento().get(this.getProducto(nCuenta).getMovimiento().size()-1));
        return retornar;
    }

    public void nuevaCuentaCorriente(String nID) {
        Producto nuevo = new CuentaCorriente();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
    }

    public void nuevaLibretaAhorro(String nID) {
        Producto nuevo = new LibretaAhorro();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
    }

    public boolean nuevaPersona(String nID, String nombre) {
        if(nID.trim().compareTo("")==0 || nombre.trim().compareTo("")==0) return false;
        else {
            Persona p=new Persona();
            p.setId(nID);
            p.setNombre(nombre);
            this.addTitular(p);
            return true;
        }
    }

    public boolean nuevaEmpresa(String nID, String nombre) {
        if(nID.trim().compareTo("")==0 || nombre.trim().compareTo("")==0) return false;
        else {
            Empresa e=new Empresa();
            e.setId(nID);
            e.setNombre(nombre);
            this.addTitular(e);
            return true;
        }
    }

    public void nuevoArqueo(int numEmp, int numCaja) {
        float saldo = this.getCaja(numCaja).getArq().getSaldoFinal();
        Arqueo ar = new Arqueo(saldo);
        this.getCaja(numCaja).setArq(ar);
        this.getCaja(numCaja).setLibre(false);
        this.getEmpleado(numEmp).setArq(ar);
    }

    public void finalizarArqueo(int numEmp, int numCaja, boolean actualizar) {
        if(actualizar) this.getCaja(numCaja).getArq().actualizarSaldoFinalComoTeorico();
        this.getCaja(numCaja).getArq().setFinalizado(true);
        this.getCaja(numCaja).setLibre(true);
        this.getEmpleado(numEmp).setArq(null);
    }

    public void nuevoPrestamoHipotecario(String nID) {
        Producto nuevo = new PrestamoHipotecario();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
    }

    public void nuevoPrestamoPersonal(String nID) {
        Producto nuevo = new PrestamoPersonal();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
    }

    public boolean reintegro(String nCuenta, float importe,int caja) {
        Calendar actual= Calendar.getInstance();
        boolean retornar = this.anadirMovimiento(nCuenta,importe*-1, "reintegro", actual.getTime());
        if(retornar)
            this.getCaja(caja).getArq().anadirMovimiento(this.getProducto(nCuenta).getMovimiento().get(this.getProducto(nCuenta).getMovimiento().size()-1));
        return retornar;
    }

    // <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
    // #[regen=yes,regenBody=yes,id=DCE.FFAA13FB-6ADF-1D40-4C07-D65505B675CB]
    // </editor-fold> 
    public void setProducto (ArrayList<Producto> val) {
        this.mProducto = val;
    }

    public ArrayList<Remesa> getRemesa () {
        return mRemesa;
    }

    public void setRemesa (ArrayList<Remesa> val) {
        this.mRemesa = val;
    }

    public ArrayList<Titular> getTitular () {
        return mTitular;
    }

    public void setTitular (ArrayList<Titular> val) {
        this.mTitular = val;
    }

    public boolean transferenciaInterna(String origen, String destino, String importe) {
        Calendar c=Calendar.getInstance();
        if(origen.compareTo(destino)!=0)
            if(this.anadirMovimiento(origen, Float.parseFloat(importe)*-1, "Transferencia interna", c.getTime()) == true) {
                this.anadirMovimiento(destino, Float.parseFloat(importe), "Transferencia interna", c.getTime());
                return true;
            }
        return false;
    }

    public boolean transferenciaExterna(String origen, String destino, String importe) {
        Calendar c=Calendar.getInstance();
        float comision=1;
        if(origen.compareTo(destino)!=0)
            if(this.anadirMovimiento(origen, (Float.parseFloat(importe)*-1)+comision, "Transferencia interna", c.getTime()) == true) {
                this.envioCECA(destino,importe,"Transferencia");
                return true;
            }
        return false;
    }

    public String verRemesa (String path) {
        String resumenRemesa = "";
        try
        {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            while ((linea = bf.readLine()) != null) {
                if (linea.substring(0 ,4).compareTo("5180")==0)
                { //leo presentador
                    resumenRemesa+=linea.substring(28, 68)+"\n";
                }
                else
                {
                   if(linea.substring(0, 4).compareTo("5680")==0)
                      resumenRemesa+=linea.substring(28, 68)+"\t"+linea.substring(88, 98)+"\n";
                }
           }
        }
        catch (IOException ex)
        {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
        return resumenRemesa;
    }

    public void incorporaRemesa (String path) throws ParseException {
        try {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            String fechaRemesa="";
            String descripcion="";
            String numCta="";
            String numCtaOrd="";
            Remesa r=new Remesa();
            while ((linea = bf.readLine()) != null) {
                String buffer;
                buffer = linea.substring(0, 4);

                if(buffer.compareTo("5180")==0){
                    //registro cabecera presentador
                    r.setOrdenante(this.getTitular(linea.substring(28,68).trim()));
                }else if(buffer.compareTo("5380")==0){
                    fechaRemesa=linea.substring(22, 28);
                    numCtaOrd = linea.substring(68,88);
                }else if(buffer.compareTo("5680")==0){
                    OperRemesa op=new OperRemesa();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    op.setCambioSaldo(n.floatValue()*-1);
                    descripcion = linea.substring(114).trim();
                    op.setDescripcion(descripcion);
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    op.setFechaOperacion(df2.parse(fechaRemesa));
                    r.addOperRemesa(op);
                    //leo el numcta
                    numCta=linea.substring(68,88);
                    this.getProducto(numCta).getMovimiento().add(op);
                } else if(linea.substring(0,4).compareTo("5681")==0) {
                    descripcion += " " + linea.substring(28,67).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5682")==0) {
                    descripcion += "\n" + linea.substring(28,79).trim() + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5683")==0) {
                    descripcion += " " + linea.substring(28,68).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5684")==0) {
                    descripcion += "\n" + linea.substring(28,79) + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5685")==0) {
                    descripcion += " " + linea.substring(28,39) + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5686")==0) {
                    descripcion += "\n" + linea.substring(28).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                }else if(buffer.compareTo("5880")==0) {
                    Movimiento mov=new Movimiento();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    mov.setCambioSaldo(n.floatValue());
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    mov.setFechaOperacion(df2.parse(fechaRemesa));
                    mov.setDescripcion("Remesa " + df2.parse(fechaRemesa).toString());
                    this.getProducto(numCtaOrd).getMovimiento().add(mov);
                }
            }
            this.mRemesa.add(r);
            this.setChanged();
            this.notifyObservers();
        } catch (IOException ex) {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Producto getProducto (String nCuenta) {
        for(int i=0;i<mProducto.size();i++)
            if(mProducto.get(i).getNumeroProducto().compareTo(nCuenta)==0)
                return mProducto.get(i);
        return null;
    }

    public int posTitularID (String ident) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getId().compareTo(ident)==0)
                return i;
        return -1;
    }

    public int posTitularNombre (String nombre) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getNombre().compareTo(nombre)==0)
                return i;
        return -1;
    }

    // <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
    // #[regen=yes,id=DCE.FFEA940E-EFB5-B736-68C1-38472BB7ADCB]
    // </editor-fold> 
    public boolean anadirMovimiento (String nCuenta, float importe, String descrip, Date fecha) {
        if(this.getProducto(nCuenta).getSaldo()+importe<0 && importe<0) {
            return false; //da error
        }
        else { //TRATAMIENTO DE ERROR. Sólo entra si control (de error) es "true"
            Movimiento nuevo=new Movimiento();
            nuevo.setCambioSaldo(importe);
            nuevo.setDescripcion(descrip);
            nuevo.setFechaOperacion(fecha);
            this.getProducto(nCuenta).getMovimiento().add(nuevo);
            this.setChanged();
            this.notifyObservers();
            return true;
        }
    }

    public String verRemesaAbono(String path) {
        String resumenRemesa = "";
        try
        {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            while ((linea = bf.readLine()) != null) {
                if (linea.substring(0 ,4).compareTo("5180")==0)
                { //leo presentador
                    resumenRemesa+=linea.substring(28, 68)+"\n";
                }
                else
                {
                   if(linea.substring(0, 4).compareTo("5680")==0)
                      resumenRemesa+=linea.substring(28, 68)+"\t"+linea.substring(88, 98)+"\n";
                }
           }
        }
        catch (IOException ex)
        {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
        return resumenRemesa;
    }

    private void envioCECA(String destino, String importe, String descripcion) {
        //hay que hacer el envio a la CECA
    }

    public ArrayList<Empleado> getMEmpleado() {
        return mEmpleado;
    }

    public void setMEmpleado(ArrayList<Empleado> mEmpleado) {
        this.mEmpleado = mEmpleado;
    }

    public ArrayList<Caja> getMCaja() {
        return mCaja;
    }

    public void setMCaja(ArrayList<Caja> mCaja) {
        this.mCaja = mCaja;
    }

    private Titular getTitular(String nombre) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getNombre().compareTo(nombre)==0)
                return mTitular.get(i);
        return null;
    }
}